@@ -1,78 +1,77 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 # -----------------------------------------------------------------------------
 #
-#  FreeType high-level python API - Copyright 2011 Nicolas P. Rougier
+#  FreeType high-level python API - Copyright 2011-2015 Nicolas P. Rougier
 #  Distributed under the terms of the new BSD license.
 #
 # -----------------------------------------------------------------------------
 '''
-FreeType high-level python API 
+FreeType high-level python API
 
 This the bindings for the high-level API of FreeType (that must be installed
 somewhere on your system).
 
-Note:
-  C Library will be searched using the ctypes.util.find_library. However, this
-  search might fail. In such a case (or for other reasons), you can specify the
-  FT_library_filename before importing the freetype library and freetype will use
-  the specified one. 
+Note: C Library will be searched using the ctypes.util.find_library. However,
+      this search might fail. In such a case (or for other reasons), you may
+      have to specify an explicit path below.
 '''
+import io
+import sys
 from ctypes import *
-from ft_types import *
-from ft_enums import *
-from ft_errors import *
-from ft_structs import *
-import ctypes.util
+
+from freetype.raw import *
+
+# Hack to get unicode class in python3
+PY3 = sys.version_info[0] == 3
+if PY3: unicode = str
+
+_handle = None
 
 
-__dll__    = None
-__handle__ = None
-FT_Library_filename = ctypes.util.find_library('freetype')
-if not FT_Library_filename:
+FT_Library_filename = filename
+
+class _FT_Library_Wrapper(FT_Library):
+    '''Subclass of FT_Library to help with calling FT_Done_FreeType'''
+    # for some reason this doesn't get carried over and ctypes complains
+    _type_ = FT_Library._type_
+
+    # Store ref to FT_Done_FreeType otherwise it will be deleted before needed.
+    _ft_done_freetype = FT_Done_FreeType
+
+    def __del__(self):
+        # call FT_Done_FreeType
+        # This does not work properly (seg fault on sime system (OSX))
+        # self._ft_done_freetype(self)
+        pass
+    
+
+def _init_freetype():
+    global _handle
+
+    _handle = _FT_Library_Wrapper()
+    error = FT_Init_FreeType( byref(_handle) )
+
+    if error: raise FT_Exception(error)
+
     try:
-        __dll__ = ctypes.CDLL('libfreetype.so.6')
-    except OSError:
-        __dll__ = None
-if not FT_Library_filename and not __dll__:
-    raise RuntimeError, 'Freetype library not found'
-if not __dll__:
-  __dll__ = ctypes.CDLL(FT_Library_filename)
-
-
+        set_lcd_filter( FT_LCD_FILTER_DEFAULT )
+    except:
+        pass
 
 # -----------------------------------------------------------------------------
 # High-level API of FreeType 2
 # -----------------------------------------------------------------------------
-FT_Init_FreeType       = __dll__.FT_Init_FreeType
-FT_Done_FreeType       = __dll__.FT_Done_FreeType
-FT_Library_Version     = __dll__.FT_Library_Version
 
-def __del_library__(self):
-    global __handle__
-    if __handle__:
-        try:
-            FT_Done_FreeType(byref(self))
-            __handle__ = None
-        except:
-            pass
-FT_Library.__del__ = __del_library__
 
 def get_handle():
     '''
     Get unique FT_Library handle
     '''
-    global __handle__
-    if not __handle__:
-        __handle__ = FT_Library( )
-        error = FT_Init_FreeType( byref(__handle__) )
-        if error: raise FT_Exception(error)
-        try:
-            set_lcd_filter( FT_LCD_FILTER_DEFAULT )
-        except:
-            pass
-        if error: raise FT_Exception(error)
-    return __handle__
+
+    if not _handle:
+        _init_freetype()
+
+    return _handle
 
 def version():
     '''
@@ -82,86 +81,12 @@ def version():
     amajor = FT_Int()
     aminor = FT_Int()
     apatch = FT_Int()
+
     library = get_handle()
     FT_Library_Version(library, byref(amajor), byref(aminor), byref(apatch))
     return (amajor.value, aminor.value, apatch.value)
 
 
-FT_Library_SetLcdFilter= __dll__.FT_Library_SetLcdFilter
-if version()>=(2,4,0):
-    FT_Library_SetLcdFilterWeights = __dll__.FT_Library_SetLcdFilterWeights
-FT_New_Face            = __dll__.FT_New_Face
-FT_New_Memory_Face     = __dll__.FT_New_Memory_Face
-FT_Open_Face           = __dll__.FT_Open_Face
-FT_Attach_File         = __dll__.FT_Attach_File
-FT_Attach_Stream       = __dll__.FT_Attach_Stream
-if version()>=(2,4,2):
-    FT_Reference_Face      = __dll__.FT_Reference_Face
-FT_Done_Face           = __dll__.FT_Done_Face
-FT_Done_Glyph          = __dll__.FT_Done_Glyph
-FT_Select_Size         = __dll__.FT_Select_Size
-FT_Request_Size        = __dll__.FT_Request_Size
-FT_Set_Char_Size       = __dll__.FT_Set_Char_Size
-FT_Set_Pixel_Sizes     = __dll__.FT_Set_Pixel_Sizes
-FT_Load_Glyph          = __dll__.FT_Load_Glyph
-FT_Load_Char           = __dll__.FT_Load_Char
-FT_Set_Transform       = __dll__.FT_Set_Transform
-FT_Render_Glyph        = __dll__.FT_Render_Glyph
-FT_Get_Kerning         = __dll__.FT_Get_Kerning
-FT_Get_Track_Kerning   = __dll__.FT_Get_Track_Kerning
-FT_Get_Glyph_Name      = __dll__.FT_Get_Glyph_Name
-FT_Get_Glyph           = __dll__.FT_Get_Glyph
-
-FT_Glyph_Get_CBox      = __dll__.FT_Glyph_Get_CBox
-
-FT_Get_Postscript_Name = __dll__.FT_Get_Postscript_Name
-FT_Get_Postscript_Name.restype = c_char_p
-FT_Select_Charmap      = __dll__.FT_Select_Charmap
-FT_Set_Charmap         = __dll__.FT_Set_Charmap
-FT_Get_Charmap_Index   = __dll__.FT_Get_Charmap_Index
-FT_Get_CMap_Language_ID= __dll__.FT_Get_CMap_Language_ID
-FT_Get_CMap_Format     = __dll__.FT_Get_CMap_Format
-FT_Get_Char_Index      = __dll__.FT_Get_Char_Index
-FT_Get_First_Char      = __dll__.FT_Get_First_Char
-FT_Get_Next_Char       = __dll__.FT_Get_Next_Char
-FT_Get_Name_Index      = __dll__.FT_Get_Name_Index
-FT_Get_SubGlyph_Info   = __dll__.FT_Get_SubGlyph_Info
-if version()>=(2,3,8):
-    FT_Get_FSType_Flags    = __dll__.FT_Get_FSType_Flags
-    FT_Get_FSType_Flags.restype  = c_ushort
-
-FT_Get_X11_Font_Format = __dll__.FT_Get_X11_Font_Format
-FT_Get_X11_Font_Format.restype = c_char_p
-
-FT_Get_Sfnt_Name_Count = __dll__.FT_Get_Sfnt_Name_Count
-FT_Get_Sfnt_Name       = __dll__.FT_Get_Sfnt_Name
-FT_Get_Advance         = __dll__.FT_Get_Advance
-
-
-FT_Outline_GetInsideBorder  = __dll__.FT_Outline_GetInsideBorder
-FT_Outline_GetOutsideBorder = __dll__.FT_Outline_GetOutsideBorder
-FT_Outline_Get_BBox         = __dll__.FT_Outline_Get_BBox
-FT_Outline_Get_CBox         = __dll__.FT_Outline_Get_CBox
-FT_Stroker_New              = __dll__.FT_Stroker_New
-FT_Stroker_Set              = __dll__.FT_Stroker_Set
-FT_Stroker_Rewind           = __dll__.FT_Stroker_Rewind
-FT_Stroker_ParseOutline     = __dll__.FT_Stroker_ParseOutline
-FT_Stroker_BeginSubPath     = __dll__.FT_Stroker_BeginSubPath
-FT_Stroker_EndSubPath       = __dll__.FT_Stroker_EndSubPath
-FT_Stroker_LineTo           = __dll__.FT_Stroker_LineTo
-FT_Stroker_ConicTo          = __dll__.FT_Stroker_ConicTo
-FT_Stroker_CubicTo          = __dll__.FT_Stroker_CubicTo
-FT_Stroker_GetBorderCounts  = __dll__.FT_Stroker_GetBorderCounts
-FT_Stroker_ExportBorder     = __dll__.FT_Stroker_ExportBorder
-FT_Stroker_GetCounts        = __dll__.FT_Stroker_GetCounts
-FT_Stroker_Export           = __dll__.FT_Stroker_Export
-FT_Stroker_Done             = __dll__.FT_Stroker_Done
-FT_Glyph_Stroke             = __dll__.FT_Glyph_Stroke
-FT_Glyph_StrokeBorder       = __dll__.FT_Glyph_StrokeBorder
-FT_Glyph_To_Bitmap          = __dll__.FT_Glyph_To_Bitmap
-
-
-
 # -----------------------------------------------------------------------------
 #  Stand alone functions
 # -----------------------------------------------------------------------------
@@ -220,8 +145,17 @@ def set_lcd_filter_weights(a,b,c,d,e):
         error = FT_Library_SetLcdFilterWeights(library, weights)
         if error: raise FT_Exception(error)
     else:
-        raise RuntimeError, \
-              'set_lcd_filter_weights require freetype > 2.4.0'
+        raise RuntimeError(
+              'set_lcd_filter_weights require freetype > 2.4.0')
+
+
+def _encode_filename(filename):
+    encoded = filename.encode(sys.getfilesystemencoding())
+    if "?" not in filename and b"?" in encoded:
+        # A bug, decoding mbcs always ignore exception, still isn't fixed in Python 2,
+        # view http://bugs.python.org/issue850997 for detail
+        raise UnicodeError()
+    return encoded
 
 
 
@@ -281,6 +215,62 @@ class BBox( object ):
 
 
 
+
+
+# -----------------------------------------------------------------------------
+class GlyphMetrics( object ):
+    '''
+
+    A structure used to model the metrics of a single glyph. The values are
+    expressed in 26.6 fractional pixel format; if the flag FT_LOAD_NO_SCALE has
+    been used while loading the glyph, values are expressed in font units
+    instead.
+
+    **Note**
+
+    If not disabled with FT_LOAD_NO_HINTING, the values represent dimensions of
+    the hinted glyph (in case hinting is applicable).
+
+    Stroking a glyph with an outside border does not increase ‘horiAdvance’ or
+    ‘vertAdvance’; you have to manually adjust these values to account for the
+    added width and height.
+    '''
+
+    def __init__(self, metrics ):
+        '''
+        Create a new GlyphMetrics object.
+
+        :param metrics: a FT_Glyph_Metrics
+        '''
+        self._FT_Glyph_Metrics = metrics
+
+    width = property( lambda self: self._FT_Glyph_Metrics.width,
+       doc = '''The glyph's width.''' )
+
+    height = property( lambda self: self._FT_Glyph_Metrics.height,
+       doc = '''The glyph's height.''' )
+
+    horiBearingX = property( lambda self: self._FT_Glyph_Metrics.horiBearingX,
+       doc = '''Left side bearing for horizontal layout.''' )
+
+    horiBearingY = property( lambda self: self._FT_Glyph_Metrics.horiBearingY,
+       doc = '''Top side bearing for horizontal layout.''' )
+
+    horiAdvance = property( lambda self: self._FT_Glyph_Metrics.horiAdvance,
+       doc = '''Advance width for horizontal layout.''' )
+
+    vertBearingX = property( lambda self: self._FT_Glyph_Metrics.vertBearingX,
+       doc = '''Left side bearing for vertical layout.''' )
+
+    vertBearingY = property( lambda self: self._FT_Glyph_Metrics.vertBearingY,
+       doc = '''Top side bearing for vertical layout. Larger positive values
+                mean further below the vertical glyph origin.''' )
+
+    vertAdvance = property( lambda self: self._FT_Glyph_Metrics.vertAdvance,
+       doc = '''Advance height for vertical layout. Positive values mean the
+                glyph has a positive advance downward.''' )
+
+
 # -----------------------------------------------------------------------------
 class SizeMetrics( object ):
     '''
@@ -457,7 +447,7 @@ class Bitmap(object):
     pixel_mode = property(lambda self: self._FT_Bitmap.pixel_mode,
            doc = '''The pixel mode, i.e., how pixel bits are stored. See
                     FT_Pixel_Mode for possible values.''')
-  
+
     palette_mode = property(lambda self: self._FT_Bitmap.palette_mode,
              doc ='''This field is intended for paletted pixel modes; it
                      indicates how the palette is stored. Not used currently.''')
@@ -570,7 +560,7 @@ class Outline( object ):
         :param charmap: a FT_Outline
         '''
         self._FT_Outline = outline
-        
+
     n_contours = property(lambda self: self._FT_Outline.n_contours)
     def _get_contours(self):
         n = self._FT_Outline.n_contours
@@ -609,7 +599,7 @@ class Outline( object ):
               argument to the SCANMODE instruction).
 
               Bits 3 and 4 are reserved for internal purposes.''')
-        
+
     flags = property(lambda self: self._FT_Outline.flags,
       doc = '''A set of bit flags used to characterize the outline and give
                hints to the scan-converter and hinter on how to
@@ -661,8 +651,7 @@ class Outline( object ):
         which is dedicated to this single task.
         '''
         bbox = FT_BBox()
-        error = FT_Outline_Get_CBox(byref(self._FT_Outline), byref(bbox))
-        if error: raise FT_Exception(error)
+        FT_Outline_Get_CBox(byref(self._FT_Outline), byref(bbox))
         return BBox(bbox)
 
 
@@ -723,6 +712,9 @@ class Glyph( object ):
                        before rendering. Can be 0 (if no translation). The origin is
                        expressed in 26.6 pixels.
 
+                       We also detect a plain vector and make a pointer out of it,
+                       if that's the case.
+
         :param destroy: A boolean that indicates that the original glyph image
                         should be destroyed by this function. It is never destroyed
                         in case of error.
@@ -738,8 +730,13 @@ class Glyph( object ):
           replaced by this function (with newly allocated data). Typically, you
           would use (omitting error handling):
         '''
-        error = FT_Glyph_To_Bitmap( byref(self._FT_Glyph),
-                                    mode, origin, destroy)
+        if ( type(origin) == FT_Vector ):
+            error = FT_Glyph_To_Bitmap( byref(self._FT_Glyph),
+                                        mode, byref(origin), destroy )
+        else:
+            error = FT_Glyph_To_Bitmap( byref(self._FT_Glyph),
+                                        mode, origin, destroy )
+
         if error: raise FT_Exception( error )
         return BitmapGlyph( self._FT_Glyph )
 
@@ -773,18 +770,18 @@ class Glyph( object ):
           can compute the width and height of the glyph image (be it in integer
           or 26.6 pixels) as:
 
-          width  = bbox.xMax - bbox.xMin;                                  
-          height = bbox.yMax - bbox.yMin;                                  
+          width  = bbox.xMax - bbox.xMin;
+          height = bbox.yMax - bbox.yMin;
 
           Note also that for 26.6 coordinates, if 'bbox_mode' is set to
           FT_GLYPH_BBOX_GRIDFIT, the coordinates will also be grid-fitted, which
           corresponds to:
 
           bbox.xMin = FLOOR(bbox.xMin);
-          bbox.yMin = FLOOR(bbox.yMin);                                    
-          bbox.xMax = CEILING(bbox.xMax);                                  
-          bbox.yMax = CEILING(bbox.yMax);                                  
-        
+          bbox.yMin = FLOOR(bbox.yMin);
+          bbox.xMax = CEILING(bbox.xMax);
+          bbox.yMax = CEILING(bbox.yMax);
+
           To get the bbox in pixel coordinates, set 'bbox_mode' to
           FT_GLYPH_BBOX_TRUNCATE.
 
@@ -792,7 +789,7 @@ class Glyph( object ):
           FT_GLYPH_BBOX_PIXELS.
         '''
         bbox = FT_BBox()
-        error = FT_Glyph_Get_CBox(byref(self._FT_Glyph), bbox_mode,byref(bbox))
+        error = FT_Glyph_Get_CBox(byref(self._FT_Glyph), bbox_mode, byref(bbox))
         if error: raise FT_Exception(error)
         return BBox(bbox)
 
@@ -887,6 +884,15 @@ class GlyphSlot( object ):
                 the bitmap buffer can change between calls of FT_Load_Glyph and
                 a few other functions.''')
 
+    def _get_metrics( self ):
+        return GlyphMetrics( self._FT_GlyphSlot.contents.metrics )
+    metrics = property( _get_metrics,
+       doc = '''The metrics of the last loaded glyph in the slot. The returned
+       values depend on the last load flags (see the FT_Load_Glyph API
+       function) and can be expressed either in 26.6 fractional pixels or font
+       units. Note that even when the glyph image is transformed, the metrics
+       are not.''')
+
     def _get_next( self ):
         return GlyphSlot( self._FT_GlyphSlot.contents.next )
     next = property( _get_next,
@@ -956,12 +962,12 @@ class Face( object ):
     FreeType root face class structure. A face object models a typeface in a
     font file.
     '''
-    def __init__( self, filename, index = 0 ):
+    def __init__( self, path_or_stream, index = 0 ):
         '''
         Build a new Face
 
-        :param str filename:
-            A path to the font file.
+        :param Union[str, typing.BinaryIO] path_or_stream:
+            A path to the font file or an io.BytesIO stream.
 
         :param int index:
                The index of the face within the font.
@@ -970,14 +976,38 @@ class Face( object ):
         library = get_handle( )
         face = FT_Face( )
         self._FT_Face = None
-        #error = FT_New_Face( library, filename, 0, byref(face) )
-        u_filename = c_char_p(filename)
-        error = FT_New_Face( library, u_filename, index, byref(face) )
-        if error: raise FT_Exception( error )
-        self._filename = filename
+        #error = FT_New_Face( library, path_or_stream, 0, byref(face) )
+        self._filebodys = []
+        if hasattr(path_or_stream, "read"):
+            error = self._init_from_memory(library, face, index, path_or_stream.read())
+        else:
+            try:
+                error = self._init_from_file(library, face, index, path_or_stream)
+            except UnicodeError:
+                with open(path_or_stream, mode="rb") as f:
+                    filebody = f.read()
+                error = self._init_from_memory(library, face, index, filebody)
+        if error:
+            raise FT_Exception(error)
         self._index = index
         self._FT_Face = face
-    
+
+    def _init_from_file(self, library, face, index, path):
+        u_filename = c_char_p(_encode_filename(path))
+        error = FT_New_Face(library, u_filename, index, byref(face))
+        return error
+
+    def _init_from_memory(self, library, face, index, byte_stream):
+        error = FT_New_Memory_Face(
+            library, byte_stream, len(byte_stream), index, byref(face)
+        )
+        self._filebodys.append(byte_stream)  # prevent gc
+        return error
+
+    @classmethod
+    def from_bytes(cls, bytes_, index=0):
+         return cls(io.BytesIO(bytes_), index)
+
     def __del__( self ):
         '''
         Discard  face object, as well as all of its child slots and sizes.
@@ -985,11 +1015,48 @@ class Face( object ):
         if self._FT_Face is not None:
             FT_Done_Face( self._FT_Face )
 
+
+    def attach_file( self, filename ):
+        '''
+        Attach data to a face object. Normally, this is used to read
+        additional information for the face object. For example, you can attach
+        an AFM file that comes with a Type 1 font to get the kerning values and
+        other metrics.
+
+        :param filename: Filename to attach
+
+        **Note**
+
+        The meaning of the 'attach' (i.e., what really happens when the new
+        file is read) is not fixed by FreeType itself. It really depends on the
+        font format (and thus the font driver).
+
+        Client applications are expected to know what they are doing when
+        invoking this function. Most drivers simply do not implement file
+        attachments.
+        '''
+
+        try:
+            u_filename = c_char_p(_encode_filename(filename))
+            error = FT_Attach_File( self._FT_Face, u_filename )
+        except UnicodeError:
+            with open(filename, mode='rb') as f:
+                filebody = f.read()
+            parameters = FT_Open_Args()
+            parameters.flags = FT_OPEN_MEMORY
+            parameters.memory_base = filebody
+            parameters.memory_size = len(filebody)
+            parameters.stream = None
+            error = FT_Attach_Stream( self._FT_Face, parameters )
+            self._filebodys.append(filebody)  # prevent gc
+        if error: raise FT_Exception( error)
+
+
     def set_char_size( self, width=0, height=0, hres=72, vres=72 ):
         '''
         This function calls FT_Request_Size to request the nominal size (in
         points).
-        
+
         :param float width: The nominal width, in 26.6 fractional points.
 
         :param float height: The nominal height, in 26.6 fractional points.
@@ -1047,9 +1114,16 @@ class Face( object ):
         '''
         Select a given charmap for character code to glyph index mapping.
 
-        :param charmap: A handle to the selected charmap.
+        :param charmap: A handle to the selected charmap, or an index to face->charmaps[]
         '''
-        error = FT_Set_Charmap( self._FT_Face, charmap._FT_Charmap )
+        if ( type(charmap) == Charmap ):
+            error = FT_Set_Charmap( self._FT_Face, charmap._FT_Charmap )
+            # Type 14 is allowed to fail, to match ft2demo's behavior.
+            if ( charmap.cmap_format == 14 ):
+                error = 0
+        else:
+            # Treat "charmap" as plain number
+            error = FT_Set_Charmap( self._FT_Face, self._FT_Face.contents.charmaps[charmap] )
         if error : raise FT_Exception(error)
 
     def get_char_index( self, charcode ):
@@ -1066,10 +1140,47 @@ class Face( object ):
           correspond to the internal indices used within the file. This is done
           to ensure that value 0 always corresponds to the 'missing glyph'.
         '''
-        if type( charcode ) in (str,unicode):
-            charcode = ord( charcode )
+        if isinstance(charcode, (str,unicode)):
+            charcode = ord(charcode)
         return FT_Get_Char_Index( self._FT_Face, charcode )
 
+    def get_glyph_name(self, agindex, buffer_max=64):
+        '''
+        This function is used to return the glyph name for the given charcode.
+
+        :param agindex: The glyph index.
+
+        :param buffer_max: The maximum number of bytes to use to store the
+            glyph name.
+
+        :param glyph_name: The glyph name, possibly truncated.
+
+        '''
+        buff = create_string_buffer(buffer_max)
+        error = FT_Get_Glyph_Name(self._FT_Face, FT_UInt(agindex), byref(buff),
+                                  FT_UInt(buffer_max))
+        if error: raise FT_Exception(error)
+        return buff.value
+
+    def get_chars( self ):
+        '''
+        This generator function is used to return all unicode character
+        codes in the current charmap of a given face. For each character it
+        also returns the corresponding glyph index.
+
+        :return: character code, glyph index
+
+        **Note**:
+          Note that 'agindex' is set to 0 if the charmap is empty. The
+          character code itself can be 0 in two cases: if the charmap is empty
+          or if the value 0 is the first valid character code.
+        '''
+        charcode, agindex = self.get_first_char()
+        yield charcode, agindex
+        while agindex != 0:
+            charcode, agindex = self.get_next_char(charcode, 0)
+            yield charcode, agindex
+
     def get_first_char( self ):
         '''
         This function is used to return the first character code in the current
@@ -1102,7 +1213,7 @@ class Face( object ):
         :param agindex: Glyph index of next character code. 0 if charmap is empty.
 
         **Note**:
-  
+
           You should use this function with FT_Get_First_Char to walk over all
           character codes available in a given charmap. See the note for this
           function for a simple code example.
@@ -1121,6 +1232,9 @@ class Face( object ):
 
         :param name: The glyph name.
         '''
+        if not isinstance(name, bytes):
+            raise FT_Exception(0x06, "FT_Get_Name_Index() expects a binary "
+                               "string for the name parameter.")
         return FT_Get_Name_Index( self._FT_Face, name )
 
     def set_transform( self, matrix, delta ):
@@ -1203,8 +1317,16 @@ class Face( object ):
           This function simply calls FT_Get_Char_Index and FT_Load_Glyph.
         '''
 
-        if len(char) == 1:
+        # python 2 with ascii input
+        if ( isinstance(char, str) and ( len(char) == 1 ) ):
             char = ord(char)
+        # python 2 with utf8 string input
+        if ( isinstance(char, str) and ( len(char) != 1 ) ):
+            char = ord(char.decode('utf8'))
+        # python 3 or python 2 with __future__.unicode_literals
+        if ( isinstance(char, unicode) and ( len(char) == 1 ) ):
+            char = ord(char)
+        # allow bare integer to pass through
         error = FT_Load_Char( self._FT_Face, char, flags )
         if error: raise FT_Exception( error )
 
@@ -1219,7 +1341,7 @@ class Face( object ):
         :param flags: A set of bit flags similar to those used when calling
                       FT_Load_Glyph, used to determine what kind of advances
                       you need.
-                    
+
         :return: The advance value, in either font units or 16.16 format.
 
                  If FT_LOAD_VERTICAL_LAYOUT is set, this is the vertical
@@ -1231,7 +1353,7 @@ class Face( object ):
         error = FT_Get_Advance( self._FT_Face, gindex, flags, byref(padvance) )
         if error: raise FT_Exception( error )
         return padvance.value
-        
+
 
 
     def get_kerning( self, left, right, mode = FT_KERNING_DEFAULT ):
@@ -1239,7 +1361,7 @@ class Face( object ):
         Return the kerning vector between two glyphs of a same face.
 
         :param left: The index of the left glyph in the kern pair.
-        
+
         :param right: The index of the right glyph in the kern pair.
 
         :param mode: See FT_Kerning_Mode for more information. Determines the scale
@@ -1267,7 +1389,7 @@ class Face( object ):
         'TrueType', 'Type 1', 'BDF', ‘PCF', ‘Type 42', ‘CID Type 1', ‘CFF',
         'PFR', and ‘Windows FNT'.
         '''
-        
+
         return FT_Get_X11_Font_Format( self._FT_Face )
 
 
@@ -1358,7 +1480,7 @@ class Face( object ):
                doc = '''True whenever a face object contains a font face that
                         contains fixed-width (or 'monospace', 'fixed-pitch',
                         etc.) glyphs.''')
-    
+
     def _has_fixed_sizes( self ):
         return bool( self.face_flags & FT_FACE_FLAG_FIXED_SIZES )
     has_fixed_sizes = property( _has_fixed_sizes,
@@ -1700,7 +1822,7 @@ class Stroker( object ):
         error = FT_Stroker_BeginSubPath( self._FT_Stroker, to, _open )
         if error: raise FT_Exception( error )
 
-    
+
     def end_subpath( self ):
         '''
         Close the current sub-path in the stroker.
@@ -1714,7 +1836,7 @@ class Stroker( object ):
         error = FT_Stroker_EndSubPath( self._FT_Stroker)
         if error: raise FT_Exception( error )
 
-    
+
     def line_to( self, to ):
         '''
         'Draw' a single line segment in the stroker's current sub-path, from
@@ -1730,7 +1852,7 @@ class Stroker( object ):
         error = FT_Stroker_LineTo( self._FT_Stroker, to )
         if error: raise FT_Exception( error )
 
-        
+
     def conic_to( self, control, to ):
         '''
         'Draw' a single quadratic Bezier in the stroker's current sub-path,
@@ -1747,7 +1869,7 @@ class Stroker( object ):
         '''
         error = FT_Stroker_ConicTo( self._FT_Stroker, control, to )
         if error: raise FT_Exception( error )
-        
+
 
     def cubic_to( self, control1, control2, to ):
         '''
@@ -1767,15 +1889,15 @@ class Stroker( object ):
         '''
         error = FT_Stroker_CubicTo( self._FT_Stroker, control1, control2, to )
         if error: raise FT_Exception( error )
-        
-        
+
+
     def get_border_counts( self, border ):
         '''
         Call this function once you have finished parsing your paths with the
         stroker. It returns the number of points and contours necessary to
         export one of the 'border' or 'stroke' outlines generated by the
         stroker.
-        
+
         :param border: The border index.
 
         :return: number of points, number of contours
@@ -1786,7 +1908,7 @@ class Stroker( object ):
                                     byref(anum_points), byref(anum_contours) )
         if error: raise FT_Exception( error )
         return anum_points.value, anum_contours.value
-                                            
+
 
     def export_border( self , border, outline ):
         '''
@@ -1799,7 +1921,7 @@ class Stroker( object ):
         :param border:  The border index.
 
         :param outline: The target outline.
-            
+
         **Note**:
 
           Always call this function after get_border_counts to get sure that
